@@ -199,7 +199,62 @@ pub(crate) fn rewrite_objects(
                                 wrapped_key: scalar_schema
                             }
                         });
-                        return; // Don't apply other logic to this promoted field
+                    }
+                } else if let Some(type_arr) = type_val.as_array() {
+                    if type_arr.len() == 2 && type_arr.contains(&Value::String("null".into())) {
+                        if let Some(inner_type) = type_arr
+                            .iter()
+                            .find(|t| *t != &Value::String("null".into()))
+                            .and_then(|t| t.as_str())
+                        {
+                            if matches!(inner_type, "string" | "integer" | "number" | "boolean") {
+                                debug!(
+                                    config,
+                                    "Force promoting nullable scalar field '{}' of type '{}'",
+                                    name,
+                                    inner_type
+                                );
+
+                                let wrapped_key = make_promoted_scalar_key(name, inner_type);
+                                let scalar_schema = schema.clone();
+
+                                *schema = serde_json::json!({
+                                    "type": "object",
+                                    "properties": {
+                                        wrapped_key: scalar_schema
+                                    }
+                                });
+                                return;
+                            }
+                        }
+                    }
+                } else if let Some(type_arr) = type_val.as_array() {
+                    if type_arr.len() == 2 && type_arr.contains(&Value::String("null".into())) {
+                        if let Some(inner_type) = type_arr
+                            .iter()
+                            .find(|t| *t != &Value::String("null".into()))
+                            .and_then(|t| t.as_str())
+                        {
+                            if matches!(inner_type, "string" | "integer" | "number" | "boolean") {
+                                debug!(
+                                    config,
+                                    "Force promoting nullable scalar field '{}' of type '{}'",
+                                    name,
+                                    inner_type
+                                );
+
+                                let wrapped_key = make_promoted_scalar_key(name, inner_type);
+                                let scalar_schema = schema.clone();
+
+                                *schema = serde_json::json!({
+                                    "type": "object",
+                                    "properties": {
+                                        wrapped_key: scalar_schema
+                                    }
+                                });
+                                return;
+                            }
+                        }
                     }
                 }
             }
@@ -284,6 +339,24 @@ pub(crate) fn rewrite_objects(
 
         // --- Heuristic rewrite ---
         if let Some(props) = obj.get("properties").and_then(|p| p.as_object()) {
+            // GUARD: Skip map conversion if this field was force-promoted to a scalar wrapper
+            if let Some(name) = field_name {
+                if config.force_scalar_promotion.contains(name) {
+                    debug!(
+                        config,
+                        "Skipping map conversion for force-promoted field '{}'", name
+                    );
+                    // Still need to recurse into properties
+                    if let Some(props_mut) =
+                        obj.get_mut("properties").and_then(|p| p.as_object_mut())
+                    {
+                        process_properties_parallel(props_mut, config, |k, v| {
+                            rewrite_objects(v, Some(k), config, false);
+                        });
+                    }
+                    return;
+                }
+            }
             // GUARD: Skip re-processing of already converted map schemas
             if obj.get("additionalProperties").is_some() {
                 if props.is_empty() {
